@@ -139,14 +139,12 @@ setAs("ggmap", "SpatRaster",
 
 
 
+as.list.SpatRaster <- function(x, ...) {
+	lapply(1:nlyr(x), function(i) x[[i]])
+}
+setMethod("as.list", signature(x="SpatRaster"), as.list.SpatRaster)
 
 
-setMethod("as.list", signature(x="SpatRaster"), 
-	function(x) {
-		lapply(1:nlyr(x), function(i) x[[i]])
-	}
-)
- 
  
 # create a "grDevices::raster" (small r) object for use with the rasterImage function
 # NOT a raster::Raster* object
@@ -220,6 +218,7 @@ setMethod("as.lines", signature(x="SpatRaster"),
 setMethod("as.polygons", signature(x="SpatExtent"), 
 	function(x, crs="") {
 		p <- methods::new("SpatVector")
+		crs <- character_crs(crs, "as.polygons")
 		p@ptr <- SpatVector$new(x@ptr, crs)
 		messages(p, "as.polygons")
 	}
@@ -227,6 +226,7 @@ setMethod("as.polygons", signature(x="SpatExtent"),
 
 setMethod("as.lines", signature(x="SpatExtent"), 
 	function(x, crs="") {
+		crs <- character_crs(crs, "lines")
 		as.lines(as.polygons(x, crs))
 	}
 )
