@@ -40,13 +40,12 @@ std::vector<double> rcValue(std::vector<double> &d, const int& nrow, const int&
 
 
 
-std::vector<double> SpatRaster::focal_values(std::vector<unsigned> w, double fillvalue, int row, int nrows) {
+std::vector<double> SpatRaster::focal_values(std::vector<unsigned> w, double fillvalue, int row, int nrows, SpatOptions &ops) {
 
 	if (nlyr() > 1) {
-		SpatOptions ops;
 		std::vector<unsigned> lyr = {0};
 		SpatRaster s = subset(lyr, ops);
-		s.focal_values(w, fillvalue, row, nrows); 
+		s.focal_values(w, fillvalue, row, nrows, ops); 
 	}
 
 	if ((w[0] % 2 == 0) || (w[1] % 2 == 0)) {
@@ -69,14 +68,15 @@ std::vector<double> SpatRaster::focal_values(std::vector<unsigned> w, double fil
 	if ((startrow+readnrows) > nr ) {
 		readnrows = nr-startrow;
 		endoff = readnrows - (nrows+startoff);
-	}	
-	std::vector<double> d = readValues(startrow, readnrows, 0, nc);
+	}
+	std::vector<double> d;
+	readValues(d, startrow, readnrows, 0, nc);
 
 //	get_focal(f, d, nrows, nc, w[0], w[1], offset, endoff, fillvalue);
 //  get_focal(std::vector<double> &out, const std::vector<double> &d, int nrow, int ncol, int wrows, int wcols, int startoff, int endoff,  double fill) {
 
 	int wc = w[1] / 2;
-	wr = std::min(wr, nrows-1);
+	wr = std::min(wr, std::max(1, nrows-1));
 	wc = std::min(wc, nc-1);
 
 	size_t n = nrows * nc * w[0] * w[1];
@@ -84,18 +84,27 @@ std::vector<double> SpatRaster::focal_values(std::vector<unsigned> w, double fil
 	int nrmax = nrows + startoff + endoff - 1;
 	//int nrmax = d.size() / ncol - 1;
 	int f = 0;
+	const bool global = is_global_lonlat();
+
 	for (int r=0; r < nrows; r++) {
 		for (int c=0; c < nc; c++) {
 			for (int i = -wr; i <= wr; i++) {
-				int row = r+startoff+i;		
+				int row = r+startoff+i;
 				if ((row < 0) || (row > nrmax)) {
-					f = f + w[1];
+					f += w[1];
 				} else {
 					unsigned bcell = row * nc;
 					for (int j = -wc; j <= wc; j++) {
 						int col = c + j;
 						if ((col >= 0) && (col < nc)) {
 							out[f] = d[bcell+col];
+						} else if (global) {
+							if (col < 0) {
+								col = nc + col;
+							} else if (col >= nc) {
+								col = col - nc;
+							} 
+							out[f] = d[bcell+col]; 
 						}
 						f++;
 					}
@@ -103,18 +112,16 @@ std::vector<double> SpatRaster::focal_values(std::vector<unsigned> w, double fil
 			}
 		}
 	}
-	
 	return out;
 }
 
 
 
-	
+
 
 void focal_win_fun(const std::vector<double> &d, std::vector<double> &out, int nc, int srow, int nr,
-                    std::vector<double> window, int wnr, int wnc, double fill, bool narm, bool expand,
-					std::function<double(std::vector<double>&, bool)> fun) {
-	
+                    std::vector<double> window, int wnr, int wnc, double fill, bool narm, bool naonly, bool naomit, bool expand, bool global, std::function<double(std::vector<double>&, bool)> fun) {
+
 	out.resize(nc * nr);
 	int hwc = wnc / 2;
 	int hwr = wnr / 2;
@@ -122,14 +129,29 @@ void focal_win_fun(const std::vector<double> &d, std::vector<double> &out, int n
 	for (size_t i=0; i<window.size(); i++) {
 		if (std::isnan(window[i])) winNA[i] = true; 
 	}
-	
+
 	int wr1 = wnr - 1;
 	int wc1 = wnc - 1;
 	int nc1 = nc - 1;
 	
+	bool checkNA = naonly || naomit;
+
 	for (int r=0; r < nr; r++) {
 		int rread = r+srow;
 		for (int c=0; c < nc; c++) {
+			size_t cell = r*nc + c;
+			if (checkNA) {
+				size_t readcell = rread*nc + c;
+				if (naonly) {
+					if (!std::isnan(d[readcell])) {
+						out[cell] = d[readcell];
+						continue;
+					}					
+				} else if (std::isnan(d[readcell])) {
+					out[cell] = d[readcell];						
+					continue;
+				}
+			}
 			std::vector<double> v;
 			v.reserve(wnr * wnc);
 			for (int rr=0; rr<wnr; rr++) {
@@ -143,7 +165,11 @@ void focal_win_fun(const std::vector<double> &d, std::vector<double> &out, int n
 					}
 					int row = rread + hwr - offr;
 					int col = c + hwc - offc;
-					if (expand) {
+					if (global) {
+						col = col < 0 ? nc + col : col;
+						col = col > nc1 ? col - nc : col;
+						v.push_back(d[nc*row + col] * window[wi]);
+					} else if (expand) {
 						col = col < 0 ? 0 : col;
 						col = col > nc1 ? nc1 : col;
 						v.push_back(d[nc*row + col] * window[wi]);
@@ -156,16 +182,16 @@ void focal_win_fun(const std::vector<double> &d, std::vector<double> &out, int n
 					}
 				}
 			}
-			out[nc * r + c] = fun(v, narm);
-		}	
+			out[cell] = fun(v, narm);
+		}
 	}
 }
 
 
 
 void focal_win_sum(const std::vector<double> &d, std::vector<double> &out, int nc, int srow, int nr,
-                    std::vector<double> window, int wnr, int wnc, double fill, bool narm, bool expand) {
-	
+                    std::vector<double> window, int wnr, int wnc, double fill, bool narm, bool naonly, bool naomit, bool expand, bool global) {
+
 	out.resize(nc*nr, NAN);
 	int hwc = wnc / 2;
 	int hwr = wnr / 2;
@@ -175,14 +201,27 @@ void focal_win_sum(const std::vector<double> &d, std::vector<double> &out, int n
 	for (size_t i=0; i<window.size(); i++) {
 		if (std::isnan(window[i])) winNA[i] = true; 
 	}
-	
+
 	int wr1 = wnr - 1;
 	int wc1 = wnc - 1;
 	int nc1 = nc - 1;
+	bool checkNA = naonly || naomit;
 
 	for (int r=0; r < nr; r++) {
 		int rread = r+srow;
 		for (int c=0; c < nc; c++) {
+			size_t cell = r*nc + c;
+			if (checkNA) {
+				size_t readcell = rread*nc + c;
+				if (naonly) {
+					if (!std::isnan(d[readcell])) {
+						out[cell] = d[readcell];
+						continue;
+					}					
+				} else if (std::isnan(d[readcell])) {
+					continue;
+				}
+			}
 			double value = 0;
 			bool found = false;
 			for (int rr=0; rr<wnr; rr++) {
@@ -196,7 +235,18 @@ void focal_win_sum(const std::vector<double> &d, std::vector<double> &out, int n
 					}
 					int row = rread + hwr - offr;
 					int col = c + hwc - offc;
-					if (expand) {
+					if (global) {
+						col = col < 0 ? nc + col : col;
+						col = col > nc1 ? col - nc : col;
+						if (narm) {
+							if (!std::isnan(d[nc * row + col])) {
+								value += d[nc*row + col] * window[wi];
+								found = true;
+							}
+						} else {
+							value += d[nc*row + col] * window[wi];
+						}
+					} else if (expand) {
 						col = col < 0 ? 0 : col;
 						col = col > nc1 ? nc1 : col;
 						if (narm) {
@@ -225,18 +275,18 @@ void focal_win_sum(const std::vector<double> &d, std::vector<double> &out, int n
 			}
 			if (narm) {
 				if (found) {
-					out[nc * r + c] = value;
+					out[cell] = value;
 				}
 			} else {
-				out[nc * r + c] = value;
+				out[cell] = value;
 			}
 		}
 	}
 }
 
 void focal_win_mean(const std::vector<double> &d, std::vector<double> &out, int nc, int srow, int nr,
-                    std::vector<double> window, int wnr, int wnc, double fill, bool narm, bool expand) {
-	
+                    std::vector<double> window, int wnr, int wnc, double fill, bool narm, bool naonly, bool naomit, bool expand, bool global) {
+
 	out.resize(nc*nr, NAN);
 	int hwc = wnc / 2;
 	int hwr = wnr / 2;
@@ -259,11 +309,24 @@ void focal_win_mean(const std::vector<double> &d, std::vector<double> &out, int
 	int wr1 = wnr - 1;
 	int wc1 = wnc - 1;
 	int nc1 = nc - 1;
-	
-	
+
+	bool checkNA = naonly || naomit;
+
 	for (int r=0; r<nr; r++) {
 		int rread = r+srow;
 		for (int c=0; c < nc; c++) {
+			size_t cell = r*nc + c;
+			if (checkNA) {
+				size_t readcell = rread*nc + c;				
+				if (naonly) {
+					if (!std::isnan(d[readcell])) {
+						out[cell] = d[readcell];
+						continue;
+					}					
+				} else if (std::isnan(d[readcell])) {
+					continue;
+				}
+			}
 			double value = 0;
 			if (narm) winsum = 0;
 			for (int rr=0; rr<wnr; rr++) {
@@ -277,7 +340,18 @@ void focal_win_mean(const std::vector<double> &d, std::vector<double> &out, int
 					}
 					int row = rread + hwr - offr;
 					int col = c + hwc - offc;
-					if (expand) {
+					if (global) {
+						col = col < 0 ? nc + col : col;
+						col = col > nc1 ? col - nc : col;
+						if (narm) {
+							if (!std::isnan(d[nc * row + col])) {
+								value += d[nc * row + col] * window[wi];
+								winsum += poswin[wi];
+							}
+						} else {
+							value += d[nc * row + col] * window[wi]; 
+						}
+					} else if (expand) {
 						col = col < 0 ? 0 : col;
 						col = col > nc1 ? nc1 : col;
 						if (narm) {
@@ -308,30 +382,20 @@ void focal_win_mean(const std::vector<double> &d, std::vector<double> &out, int
 				}
 			}
 			if (winsum > 0) {
-				out[nc * r + c] = value / winsum;
-			}	
-		}	
+				out[cell] = value / winsum;
+			}
+		}
 	}
 }
 
 
 
 
-
-SpatRaster SpatRaster::focal3(std::vector<unsigned> w, std::vector<double> m, double fillvalue, bool narm, bool naonly, std::string fun, bool expand, SpatOptions &opt) {
+SpatRaster SpatRaster::focal(std::vector<unsigned> w, std::vector<double> m, double fillvalue, bool narm, bool naonly, bool naomit, std::string fun, bool expand, SpatOptions &opt) {
 
 	SpatRaster out = geometry();
+	bool global = is_global_lonlat();
 	size_t nl = nlyr();
-/*
-	SpatRaster out = geometry(1);
-	if (nlyr() > 1) {
-		SpatOptions ops(opt);
-		out.addWarning("focal computations are only done for the first layer");
-		std::vector<unsigned> lyr = {0};
-		SpatRaster s = subset(lyr, ops);
-		s.focal3(w, m, fillvalue, narm, naonly, fun, expand, opt);
-	}
-*/
 
 	if (!source[0].hasValues) { return(out); }
 
@@ -343,11 +407,6 @@ SpatRaster SpatRaster::focal3(std::vector<unsigned> w, std::vector<double> m, do
 		out.setError("w must be odd sized");
 		return out;
 	}
-	// perhaps we can do (1,3) or (3,1)
-	//if (w[0] < 3 && w[1] < 3) {
-	//	out.setError("w must be > 1");
-	//	return out;
-	//}
 	unsigned ww = w[0] * w[1];
 	if (ww < 3) {
 		out.setError("not a meanigful window");
@@ -370,14 +429,13 @@ SpatRaster SpatRaster::focal3(std::vector<unsigned> w, std::vector<double> m, do
 		return out;
 	}
 
-
 	if (!readStart()) {
 		out.setError(getError());
 		return(out);
 	}
 	opt.ncopies += 2;
 	opt.minrows = w[0] > nr ? nr : w[0];
-	
+
  	if (!out.writeStart(opt)) {
 		readStop();
 		return out;
@@ -397,20 +455,20 @@ SpatRaster SpatRaster::focal3(std::vector<unsigned> w, std::vector<double> m, do
 		fFun = getFun(fun);
 		dofun = true;
 	}
-	
+
 	std::vector<double> fill;
 	for (size_t i = 0; i < out.bs.n; i++) {
 		unsigned rstart, roff;
 		unsigned rnrows = out.bs.nrows[i];
 		if (i == 0) {
 			rstart = 0;
-			roff = dhw0;	
+			roff = dhw0;
 			if (i != (out.bs.n-1)) {
 				rnrows += hw0;
 			}
 		} else {
 			rstart = out.bs.row[i] + hw0;
-			roff = hw0;	
+			roff = hw0;
 			if (i == (out.bs.n-1)) {
 				rnrows -= hw0; 
 			}
@@ -420,10 +478,11 @@ SpatRaster SpatRaster::focal3(std::vector<unsigned> w, std::vector<double> m, do
 		std::vector<double> vin, vincomb;
 		size_t off=0;
 		if (nl > 1) {
-			vincomb = readValues(rstart, rnrows, 0, nc);
+			readValues(vincomb, rstart, rnrows, 0, nc);
 			off = nc * out.bs.nrows[i];
+			voutcomb.reserve(vincomb.size());
 		} else {
-			vin = readValues(rstart, rnrows, 0, nc);
+			readValues(vin, rstart, rnrows, 0, nc);
 		}
 		for (size_t lyr=0; lyr<nl; lyr++) {
 			vout.clear();
@@ -440,9 +499,9 @@ SpatRaster SpatRaster::focal3(std::vector<unsigned> w, std::vector<double> m, do
 				} else {
 					fill.resize(fsz2, fillvalue);
 				}
-			}			
+			}
 			vin.insert(vin.begin(), fill.begin(), fill.end());
-			
+
 			if (i == (out.bs.n-1)) {
 				if (expand) {
 					fill = {vin.end()-nc, vin.end()};
@@ -456,33 +515,33 @@ SpatRaster SpatRaster::focal3(std::vector<unsigned> w, std::vector<double> m, do
 			}
 
 			if (dofun) {
-				focal_win_fun(vin, vout, nc, roff, out.bs.nrows[i], m, w[0], w[1], fillvalue, narm, expand, fFun);			
+				focal_win_fun(vin, vout, nc, roff, out.bs.nrows[i], m, w[0], w[1], fillvalue, narm, naonly, naomit, expand, global, fFun);
 			} else if (fun == "mean") {
-				focal_win_mean(vin, vout, nc, roff, out.bs.nrows[i], m, w[0], w[1], fillvalue, narm, expand);
+				focal_win_mean(vin, vout, nc, roff, out.bs.nrows[i], m, w[0], w[1], fillvalue, narm, naonly, naomit, expand, global);
 			} else {
-				focal_win_sum(vin, vout, nc, roff, out.bs.nrows[i], m, w[0], w[1], fillvalue, narm, expand);
+				focal_win_sum(vin, vout, nc, roff, out.bs.nrows[i], m, w[0], w[1], fillvalue, narm, naonly, naomit, expand, global);
 			}
-			
+
 			if (i != (out.bs.n-1)) {
 				fill = {vin.end() - fsz2, vin.end() };  
 			}
-		
+
 			if (naonly) {
 				for (size_t j=0; j<vout.size(); j++) {
 					size_t k = fsz2 + j;
 					if (!std::isnan(vin[k])) {
-						vout[j] = vin[k];	
+						vout[j] = vin[k];
 					}
 				}
 			}
 			if (nl > 1) {
 				voutcomb.insert(voutcomb.end(), vout.begin(), vout.end());
-			}	
+			}
 		}
 		if (nl > 1) {
-			if (!out.writeValues(voutcomb, out.bs.row[i], out.bs.nrows[i], 0, nc)) return out;			
+			if (!out.writeBlock(voutcomb, i)) return out;
 		} else {
-			if (!out.writeValues(vout, out.bs.row[i], out.bs.nrows[i], 0, nc)) return out;
+			if (!out.writeBlock(vout, i)) return out;
 		}
 	}
 
@@ -528,7 +587,7 @@ SpatRaster SpatRaster::focal1(std::vector<unsigned> w, std::vector<double> m, do
 		out.setError("w must be > 1");
 		return out;
 	}
-	
+
 	unsigned ww = w[0] * w[1];
 	if (ww < 9) {
 		out.setError("not a meaningful window");
@@ -579,12 +638,12 @@ SpatRaster SpatRaster::focal1(std::vector<unsigned> w, std::vector<double> m, do
 					}
 				}
 			}
-			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i], 0, ncol())) return out;
+			if (!out.writeBlock(v, i)) return out;
 		}
 	} else {
 		for (size_t i = 0; i < out.bs.n; i++) {
-	
-	
+
+
 			std::vector<double> fv = focal_values(w, fillvalue, out.bs.row[i], out.bs.nrows[i]);
 			v.resize(out.bs.nrows[i] * ncol());
 			if (wmat) {
@@ -616,7 +675,7 @@ SpatRaster SpatRaster::focal1(std::vector<unsigned> w, std::vector<double> m, do
 					v[j] = fFun(x, narm);
 				}
 			}
-			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i], 0, ncol())) return out;
+			if (!out.writeBlock(v, i)) return out;
 		}
 	}
 	out.writeStop();
@@ -702,7 +761,7 @@ SpatRaster SpatRaster::focal2(std::vector<unsigned> w, std::vector<double> m, do
 	}
 	opt.ncopies += std::max(1, (int)(nc * ww / ncell()));
 	opt.minrows = w[0] > nr ? nr : w[0];
-	
+
  	if (!out.writeStart(opt)) {
 		readStop();
 		return out;
@@ -714,22 +773,22 @@ SpatRaster SpatRaster::focal2(std::vector<unsigned> w, std::vector<double> m, do
 	std::vector<double> fill(hw0, fillvalue);
 
 	std::vector<double> fvals(nc * ww, fillvalue);
-	
+
 	std::vector<double> vout;
 	for (size_t i = 0; i < out.bs.n; i++) {
 		if (i==0) {
 			if (i == (out.bs.n-1)) {
-				vout.resize(out.bs.nrows[i]* nc);				
+				vout.resize(out.bs.nrows[i]* nc);
 			} else {
 				vout.resize((out.bs.nrows[i]-hw0) * nc);
 			}
 		} else if (i == (out.bs.n-1)) {
 			vout.resize((out.bs.nrows[i] + 2 * hw0)* nc);
 		} else if (i == 1) {
-			vout.resize(out.bs.nrows[i]* nc);							
+			vout.resize(out.bs.nrows[i]* nc);
 		}
-		
-		std::vector<double> vin = readValues(out.bs.row[i], out.bs.nrows[i], 0, nc);
+
+		std::vector<double> vin = readValues(out.bs.row[i], out.bs.nrows[i]);
 		size_t start= 0;
 		if (i == 0) {
 			for (size_t r=0; r<hw0; r++) {
@@ -758,7 +817,7 @@ SpatRaster SpatRaster::focal2(std::vector<unsigned> w, std::vector<double> m, do
 				}
 			}
 		}
-		
+
 		if (i == (out.bs.n - 1)) {
 			size_t m = i == 0 ? 1 : 2;
 			std::vector<double> row(nc, fillvalue);
@@ -778,7 +837,7 @@ SpatRaster SpatRaster::focal2(std::vector<unsigned> w, std::vector<double> m, do
 						vout[off+j] = fFun(x, narm);
 					}
 				}
-			}	
+			}
 		}
 
 		int startrow = out.bs.row[i]; 
@@ -791,9 +850,9 @@ SpatRaster SpatRaster::focal2(std::vector<unsigned> w, std::vector<double> m, do
 			startrow -= hw0;
 			nrows += hw0;
 		} else {
-			startrow -= hw0;			
+			startrow -= hw0;
 		}
-		if (!out.writeValues(vout, startrow, nrows, 0, nc)) return out;		
+		if (!out.writeValues(vout, startrow, nrows, 0, nc)) return out;
 	}
 	out.writeStop();
 	readStop();
