@@ -1259,6 +1259,68 @@ std::vector<int> SpatVector::relate(std::string relation, bool symmetrical) {
 
 
 
+std::vector<bool> SpatVector::is_related(SpatVector v, std::string relation) {
+
+	std::vector<bool> out;
+	int pattern = getRel(relation);
+	if (pattern == 2) {
+		setError("'" + relation + "'" + " is not a valid relate name or pattern");
+		return out;
+	}
+
+	GEOSContextHandle_t hGEOSCtxt = geos_init();
+	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
+	std::vector<GeomPtr> y = geos_geoms(&v, hGEOSCtxt);
+	size_t nx = size();
+	size_t ny = v.size();
+	out.resize(nx, false);
+	if (pattern == 1) {
+		for (size_t i = 0; i < nx; i++) {
+			for (size_t j = 0; j < ny; j++) {
+				bool isrel = GEOSRelatePattern_r(hGEOSCtxt, x[i].get(), y[j].get(), relation.c_str());
+				if (isrel) {
+					out[i] = true;
+					continue;
+				}
+			}
+		}
+	} else {
+		std::function<char(GEOSContextHandle_t, const GEOSPreparedGeometry *, const GEOSGeometry *)> relFun = getPrepRelateFun(relation);
+		for (size_t i = 0; i < nx; i++) {
+			PrepGeomPtr pr = geos_ptr(GEOSPrepare_r(hGEOSCtxt, x[i].get()), hGEOSCtxt);
+			for (size_t j = 0; j < ny; j++) {
+				bool isrel = relFun(hGEOSCtxt, pr.get(), y[j].get());
+				if (isrel) {
+					out[i] = true;
+					continue;
+				}				
+			}
+		} 
+	}
+	geos_finish(hGEOSCtxt);
+
+	return out;
+}
+
+
+
+SpatVector SpatVector::mask(SpatVector x, bool inverse) {
+	std::vector<bool> b = is_related(x, "intersects");
+	if (inverse) {
+		for (size_t i=0; i<b.size(); i++) {
+			b[i] = !b[i];
+		}
+	} 
+	std::vector<int> r;
+	r.reserve(b.size());
+	for (size_t i=0; i<b.size(); i++) {
+		if (b[i]) r.push_back(i);
+	}
+	return subset_rows(r);	
+}
+
+
+
 std::vector<double> SpatVector::geos_distance(SpatVector v, bool parallel) {
 
 	std::vector<double> out;
@@ -1387,15 +1449,31 @@ std::vector<double> SpatVector::geos_distance(bool sequential) {
 
 
 SpatVector SpatVector::unite(SpatVector v) {
+	if (type() != v.type()) {
+		SpatVector out;
+		out.setError("cannot unite different geom types");
+		return out;
+	}
+	
 	SpatVector intsec = intersect(v);
 	if (intsec.hasError()) {
 		return intsec;
 	}
+	if (intsec.nrow() == 0) {
+		return append(v, true);
+	}
+
 	SpatVector sdif = symdif(v);
 	if (sdif.hasError()) {
 		return sdif;
 	}
-	return intsec.append(sdif, true);
+	
+	if (sdif.type() == type()) {
+		return intsec.append(sdif, true);
+	} else {
+		SpatVector out;
+		return out;		
+	}
 }
 
 
@@ -1435,11 +1513,18 @@ SpatVector SpatVector::unite() {
 SpatVector SpatVector::symdif(SpatVector v) {
 	if ((type() != "polygons") || (v.type() != "polygons")) {
 		SpatVector out;
-		out.setError("expect two polygon geometries");
+		out.setError("expected two polygon geometries");
 		return out;
 	}
 	SpatVector out = erase(v);
-	out = out.append(v.erase(*this), true);
+	if (out.hasError()) {
+		return out;
+	}
+	SpatVector ve = v.erase(*this);
+	if (ve.hasError()) {
+		return ve;
+	}
+	out = out.append(ve, true);
 	return out;
 
 /*
@@ -1557,7 +1642,8 @@ SpatVector SpatVector::erase_agg(SpatVector v) {
 		out.srs = srs;
 		out.df = df.subset_rows(rids);
 	} else {
-		out = subset_rows({-1});
+		std::vector<int> none(1, -1);
+		out = subset_rows(none);
 	}
 	geos_finish(hGEOSCtxt);
 	if (!srs.is_same(v.srs, true)) {
@@ -1614,11 +1700,12 @@ SpatVector SpatVector::erase(SpatVector v) {
 		out = coll.get(0);
 		out.srs = srs;
 		out.df = df;
-		if (rids.size() != nx) {
+		if (rids.size() != out.nrow()) {
 			out = out.subset_rows(rids);
 		}
 	} else {
-		out = subset_rows({-1});		
+		std::vector<int> none(1, -1);
+		out = subset_rows(none);		
 	}
 	geos_finish(hGEOSCtxt);
 	if (!srs.is_same(v.srs, true)) {
